@@ -1,27 +1,10 @@
 #![deny(missing_docs)]
-//! Debian version type, consistent with Section 5.6.12 in the Debian Policy Manual
-//!
-//! This structure can be used for validating, dissecting and comparing Debian version strings.
-//!
-//! # Examples
-//!
-//! ```
-//! use debversion::Version;
-//!
-//! let version1: Version = "1.2.3".parse().unwrap();
-//! assert_eq!(version1.upstream_version.as_str(), "1.2.3");
-//! assert_eq!(version1.debian_revision, None);
-//! assert_eq!(version1.epoch, None);
-//!
-//! let version2: Version = "1:1.2.3".parse().unwrap();
-//! assert_eq!(version2.upstream_version.as_str(), "1.2.3");
-//! assert_eq!(version2.debian_revision, None);
-//! assert_eq!(version2.epoch, Some(1));
-//!
-//! assert_eq!(version1, version1);
-//! assert!(version1 < version2);
+#![doc = include_str!(concat!(env!("CARGO_MANIFEST_DIR"), "/README.md"))]
+// Until we drop support for PyO3 0.22
+#![allow(deprecated)]
 
 use lazy_regex::{regex_captures, regex_replace};
+use num_bigint::BigInt;
 use std::cmp::Ordering;
 use std::str::FromStr;
 
@@ -54,21 +37,16 @@ fn non_digit_cmp(va: &str, vb: &str) -> Ordering {
         }
     }
 
-    let la: Vec<i32> = va.chars().map(order).collect();
-    let lb: Vec<i32> = vb.chars().map(order).collect();
-    let mut la_iter = la.iter();
-    let mut lb_iter = lb.iter();
-    while la_iter.len() > 0 || lb_iter.len() > 0 {
-        let a = if let Some(a) = la_iter.next() { *a } else { 0 };
-        let b = if let Some(b) = lb_iter.next() { *b } else { 0 };
-        if a < b {
-            return Ordering::Less;
-        }
-        if a > b {
-            return Ordering::Greater;
-        }
-    }
-    Ordering::Equal
+    va.chars()
+        .map(order)
+        .chain(std::iter::repeat(0))
+        .zip(vb.chars().map(order).chain(std::iter::repeat(0)))
+        .take(va.len().max(vb.len()))
+        .find_map(|(a, b)| match a.cmp(&b) {
+            Ordering::Equal => None,
+            other => Some(other),
+        })
+        .unwrap_or(Ordering::Equal)
 }
 
 #[test]
@@ -86,14 +64,21 @@ fn test_non_digit_cmp() {
     assert_eq!(non_digit_cmp("~~", "~~a"), Ordering::Less);
     assert_eq!(non_digit_cmp("~~a", "~"), Ordering::Less);
     assert_eq!(non_digit_cmp("~", "a"), Ordering::Less);
+    // Test special characters that exercise the arithmetic on line 36
+    assert_eq!(non_digit_cmp("!", "@"), Ordering::Less); // ! = 33, @ = 64 + 256 = 320
+    assert_eq!(non_digit_cmp("@", "!"), Ordering::Greater);
+    assert_eq!(non_digit_cmp("#", "$"), Ordering::Less); // # = 35, $ = 36
+    assert_eq!(non_digit_cmp("|", "}"), Ordering::Less); // | = 124, } = 125
 }
 
-fn drop_leading_zeroes(mut s: &str) -> &str {
+fn drop_leading_zeroes(s: &str) -> &str {
     // Drop leading zeroes while the next character is a digit
-    while s.starts_with('0') && s.chars().nth(1).map_or(false, |c| c.is_ascii_digit()) {
-        s = &s[1..];
+    let bytes = s.as_bytes();
+    let mut start = 0;
+    while start + 1 < bytes.len() && bytes[start] == b'0' && bytes[start + 1].is_ascii_digit() {
+        start += 1;
     }
-    s
+    &s[start..]
 }
 
 fn version_cmp_part(mut a: &str, mut b: &str) -> Ordering {
@@ -128,20 +113,30 @@ fn version_cmp_part(mut a: &str, mut b: &str) -> Ordering {
             .position(|c| !c.is_ascii_digit())
             .unwrap_or(b.len())];
 
-        let a_num = if a_digit.is_empty() {
-            0
-        } else {
-            a_digit.parse::<i32>().unwrap()
-        };
-
-        let b_num = if b_digit.is_empty() {
-            0
-        } else {
-            b_digit.parse::<i32>().unwrap()
-        };
-
         // Compare the digit part
-        match a_num.cmp(&b_num) {
+        let ordering = match (a_digit.len(), b_digit.len()) {
+            (0, 0) => Ordering::Equal,
+            (0, _) => Ordering::Less,
+            (_, 0) => Ordering::Greater,
+            // For small numbers that fit in u64, avoid BigInt allocation
+            (a_len, b_len) if a_len <= 19 && b_len <= 19 => {
+                match (a_digit.parse::<u64>(), b_digit.parse::<u64>()) {
+                    (Ok(a_num), Ok(b_num)) => a_num.cmp(&b_num),
+                    // Fallback to BigInt if parsing fails (shouldn't happen with valid version strings)
+                    _ => a_digit
+                        .parse::<BigInt>()
+                        .unwrap()
+                        .cmp(&b_digit.parse::<BigInt>().unwrap()),
+                }
+            }
+            // For very long digit sequences, use BigInt
+            _ => a_digit
+                .parse::<BigInt>()
+                .unwrap()
+                .cmp(&b_digit.parse::<BigInt>().unwrap()),
+        };
+
+        match ordering {
             Ordering::Equal => (),
             ordering => return ordering,
         }
@@ -217,12 +212,16 @@ impl FromStr for Version {
             })
             .transpose()?;
 
-        let debian_revision = Some(debian_revision).filter(|e| !e.is_empty());
+        let debian_revision = if debian_revision.is_empty() {
+            None
+        } else {
+            Some(debian_revision.to_string())
+        };
 
         Ok(Version {
             epoch,
             upstream_version: upstream_version.to_string(),
-            debian_revision: debian_revision.map(|e| e.to_string()),
+            debian_revision,
         })
     }
 }
@@ -264,6 +263,96 @@ impl Version {
         )
     }
 
+    /// Is this a binNMU?
+    ///
+    /// A binNMU is a binary-only NMU (Non-Maintainer Upload) where the source package is not
+    /// changed.
+    ///
+    /// Note that this checks for the presence of the `+b[:digit:]` suffix, which is not part of the Debian
+    /// Policy Manual, but it is commonly used to indicate a binNMU.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use debversion::Version;
+    /// assert!("1.0+b1".parse::<Version>().unwrap().is_bin_nmu());
+    /// assert!("1.0-1+b1".parse::<Version>().unwrap().is_bin_nmu());
+    /// assert!(!"1.0-1".parse::<Version>().unwrap().is_bin_nmu());
+    /// assert!(!"1.0".parse::<Version>().unwrap().is_bin_nmu());
+    /// ```
+    pub fn is_bin_nmu(&self) -> bool {
+        self.bin_nmu_count().is_some()
+    }
+
+    /// Return the binNMU count of this version
+    ///
+    /// This will return the binNMU count of this version, or None if this is not a binNMU.
+    pub fn bin_nmu_count(&self) -> Option<i32> {
+        fn bin_nmu_suffix(s: &str) -> Option<i32> {
+            s.split_once("+b").and_then(|(_, rest)| rest.parse().ok())
+        }
+        if let Some(debian_revision) = self.debian_revision.as_ref() {
+            bin_nmu_suffix(debian_revision)
+        } else {
+            bin_nmu_suffix(self.upstream_version.as_str())
+        }
+    }
+
+    /// Create a binNMU version from this version
+    ///
+    /// This will increment the binNMU suffix by one, or add a `+b1` suffix if there is no binNMU
+    /// suffix.
+    pub fn increment_bin_nmu(self) -> Version {
+        fn increment_bin_nmu_suffix(s: &str) -> String {
+            match s.split_once("+b") {
+                Some((prefix, rest)) => match rest.parse::<i32>() {
+                    Ok(num) => format!("{}+b{}", prefix, num + 1),
+                    Err(_) => format!("{}+b1", s),
+                },
+                None => format!("{}+b1", s),
+            }
+        }
+
+        if let Some(debian_revision) = self.debian_revision.as_ref() {
+            Version {
+                epoch: self.epoch,
+                upstream_version: self.upstream_version,
+                debian_revision: Some(increment_bin_nmu_suffix(debian_revision)),
+            }
+        } else {
+            Version {
+                epoch: self.epoch,
+                upstream_version: increment_bin_nmu_suffix(&self.upstream_version),
+                debian_revision: self.debian_revision,
+            }
+        }
+    }
+
+    /// Check if this version is a sourceful NMU
+    ///
+    /// A sourceful NMU is a Non-Maintainer Upload where the source package is changed.
+    /// This is indicated by the presence of a `+nmu[:digit:]` suffix.
+    /// This is not part of the Debian Policy Manual, but it is commonly used to indicate a
+    /// sourceful NMU.
+    pub fn is_nmu(&self) -> bool {
+        self.nmu_count().is_some()
+    }
+
+    /// Return the sourceful NMU count of this version
+    ///
+    /// This will return the sourceful NMU count of this version, or None if this is not a
+    /// sourceful NMU.
+    pub fn nmu_count(&self) -> Option<i32> {
+        fn nmu_suffix(s: &str) -> Option<i32> {
+            s.split_once("+nmu").and_then(|(_, rest)| rest.parse().ok())
+        }
+        if let Some(debian_revision) = self.debian_revision.as_ref() {
+            nmu_suffix(debian_revision)
+        } else {
+            nmu_suffix(self.upstream_version.as_str())
+        }
+    }
+
     /// Return canonicalized version of this version
     ///
     /// # Examples
@@ -275,24 +364,34 @@ impl Version {
     /// ```
     pub fn canonicalize(&self) -> Version {
         let epoch = match self.epoch {
-            Some(0) | None => None,
-            Some(epoch) => Some(epoch),
+            Some(0) => None,
+            epoch => epoch,
         };
 
-        let mut upstream_version = self.upstream_version.as_str();
-
-        upstream_version = drop_leading_zeroes(upstream_version);
+        let upstream_version_stripped = drop_leading_zeroes(&self.upstream_version);
+        let upstream_version = if upstream_version_stripped == self.upstream_version {
+            self.upstream_version.clone()
+        } else {
+            upstream_version_stripped.to_string()
+        };
 
         let debian_revision = match self.debian_revision.as_ref() {
             Some(r) if r.chars().all(|c| c == '0') => None,
             None => None,
-            Some(revision) => Some(drop_leading_zeroes(revision)),
+            Some(revision) => {
+                let stripped = drop_leading_zeroes(revision);
+                if stripped == revision {
+                    Some(revision.clone())
+                } else {
+                    Some(stripped.to_string())
+                }
+            }
         };
 
         Version {
             epoch,
-            upstream_version: upstream_version.to_string(),
-            debian_revision: debian_revision.map(|r| r.to_string()),
+            upstream_version,
+            debian_revision,
         }
     }
 
@@ -301,22 +400,16 @@ impl Version {
     /// For native packages, increment the upstream version number.
     /// For other packages, increment the debian revision.
     pub fn increment_debian(&mut self) {
-        if self.debian_revision.is_some() {
-            self.debian_revision = self.debian_revision.as_ref().map(|v| {
-                {
-                    regex_replace!(r"\d+$", v, |x: &str| (x.parse::<i32>().unwrap() + 1)
-                        .to_string())
-                }
-                .to_string()
-            });
+        if let Some(ref mut debian_revision) = self.debian_revision {
+            *debian_revision = regex_replace!(r"\d+$", debian_revision, |x: &str| {
+                (x.parse::<i32>().unwrap() + 1).to_string()
+            })
+            .into_owned();
         } else {
-            self.upstream_version =
-                regex_replace!(r"\d+$", self.upstream_version.as_ref(), |x: &str| (x
-                    .parse::<i32>()
-                    .unwrap()
-                    + 1)
-                .to_string())
-                .to_string();
+            self.upstream_version = regex_replace!(r"\d+$", &self.upstream_version, |x: &str| {
+                (x.parse::<i32>().unwrap() + 1).to_string()
+            })
+            .into_owned();
         }
     }
 
@@ -342,7 +435,8 @@ impl sqlx::Encode<'_, Postgres> for Version {
         &self,
         buf: &mut sqlx::postgres::PgArgumentBuffer,
     ) -> Result<sqlx::encode::IsNull, Box<dyn std::error::Error + Send + Sync>> {
-        sqlx::Encode::<Postgres>::encode_by_ref(&self.to_string().as_str(), buf)
+        let version_str = self.to_string();
+        sqlx::Encode::<Postgres>::encode_by_ref(&version_str.as_str(), buf)
     }
 }
 
@@ -374,7 +468,7 @@ use pyo3::prelude::*;
 #[cfg(feature = "python-debian")]
 impl FromPyObject<'_> for Version {
     fn extract_bound(ob: &Bound<PyAny>) -> PyResult<Self> {
-        let debian_support = Python::import_bound(ob.py(), "debian.debian_support")?;
+        let debian_support = Python::import(ob.py(), "debian.debian_support")?;
         let version_cls = debian_support.getattr("Version")?;
         if !ob.is_instance(&version_cls)? {
             return Err(pyo3::exceptions::PyTypeError::new_err("Expected a Version"));
@@ -391,21 +485,17 @@ impl FromPyObject<'_> for Version {
 }
 
 #[cfg(feature = "python-debian")]
-impl ToPyObject for Version {
-    fn to_object(&self, py: Python) -> PyObject {
-        let debian_support = py.import_bound("debian.debian_support").unwrap();
-        let version_cls = debian_support.getattr("Version").unwrap();
-        version_cls
-            .call1((self.to_string(),))
-            .unwrap()
-            .to_object(py)
-    }
-}
+impl<'py> IntoPyObject<'py> for Version {
+    type Target = PyAny;
 
-#[cfg(feature = "python-debian")]
-impl IntoPy<PyObject> for Version {
-    fn into_py(self, py: Python) -> PyObject {
-        self.to_object(py)
+    type Output = Bound<'py, Self::Target>;
+
+    type Error = PyErr;
+
+    fn into_pyobject(self, py: Python<'py>) -> Result<Self::Output, Self::Error> {
+        let debian_support = py.import("debian.debian_support").unwrap();
+        let version_cls = debian_support.getattr("Version").unwrap();
+        version_cls.call1((self.to_string(),))
     }
 }
 
@@ -415,17 +505,22 @@ mod python_tests {
     fn test_from_pyobject() {
         use super::Version;
         use pyo3::prelude::*;
+        use std::ffi::CString;
 
         Python::with_gil(|py| {
-            let globals = pyo3::types::PyDict::new_bound(py);
+            let globals = pyo3::types::PyDict::new(py);
             globals
                 .set_item(
                     "debian_support",
-                    py.import_bound("debian.debian_support").unwrap(),
+                    py.import("debian.debian_support").unwrap(),
                 )
                 .unwrap();
             let v = py
-                .eval_bound("debian_support.Version('1.0-1')", Some(&globals), None)
+                .eval(
+                    &CString::new("debian_support.Version('1.0-1')").unwrap(),
+                    Some(&globals),
+                    None,
+                )
                 .unwrap()
                 .extract::<Version>()
                 .unwrap();
@@ -451,10 +546,26 @@ mod python_tests {
                 upstream_version: "1.0".to_string(),
                 debian_revision: Some("1".to_string()),
             };
-            let v = v.to_object(py);
+            let v = v.into_pyobject(py).unwrap();
             let expected: Version = "1:1.0-1".parse().unwrap();
-            assert_eq!(v.extract::<Version>(py).unwrap(), expected);
-            assert_eq!(v.bind(py).get_type().name().unwrap(), "Version");
+            assert_eq!(v.get_type().name().unwrap(), "Version");
+            assert_eq!(v.unbind().extract::<Version>(py).unwrap(), expected);
+        });
+    }
+
+    #[test]
+    fn test_from_pyobject_error() {
+        use super::Version;
+        use pyo3::prelude::*;
+        use std::ffi::CString;
+
+        Python::with_gil(|py| {
+            // Test that extracting from a non-Version object fails
+            let string_obj = py
+                .eval(&CString::new("'not a version'").unwrap(), None, None)
+                .unwrap();
+            let result = string_obj.extract::<Version>();
+            assert!(result.is_err());
         });
     }
 }
@@ -500,7 +611,7 @@ impl AsVersion for String {
 
 impl AsVersion for Version {
     fn into_version(self) -> Result<Version, ParseError> {
-        Ok(self.clone())
+        Ok(self)
     }
 }
 
@@ -644,6 +755,13 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_parse_error_display() {
+        let error = ParseError("test error message".to_string());
+        assert_eq!(format!("{}", error), "test error message");
+        assert_eq!(error.to_string(), "test error message");
+    }
+
     #[test]
     fn test_to_string() {
         assert_eq!(
@@ -760,4 +878,212 @@ mod tests {
         assert!("1.0".parse::<Version>().unwrap().is_native());
         assert!(!"1.0-0".parse::<Version>().unwrap().is_native());
     }
+
+    #[test]
+    fn test_is_binnmu() {
+        assert!("1.0+b1".parse::<Version>().unwrap().is_bin_nmu());
+        assert!("1.0-1+b1".parse::<Version>().unwrap().is_bin_nmu());
+        assert!(!"1.0-1".parse::<Version>().unwrap().is_bin_nmu());
+        assert!(!"1.0".parse::<Version>().unwrap().is_bin_nmu());
+    }
+
+    #[test]
+    fn test_bin_nmu_count() {
+        assert_eq!(
+            Some(1),
+            "1.0+b1".parse::<Version>().unwrap().bin_nmu_count()
+        );
+        assert_eq!(
+            Some(1),
+            "1.0-1+b1".parse::<Version>().unwrap().bin_nmu_count()
+        );
+        assert_eq!(None, "1.0-1".parse::<Version>().unwrap().bin_nmu_count());
+        assert_eq!(None, "1.0".parse::<Version>().unwrap().bin_nmu_count());
+    }
+
+    #[test]
+    fn test_increment_bin_nmu() {
+        assert_eq!(
+            "1.0+b2".parse::<Version>().unwrap(),
+            "1.0+b1".parse::<Version>().unwrap().increment_bin_nmu()
+        );
+        assert_eq!(
+            "1.0-1+b2".parse::<Version>().unwrap(),
+            "1.0-1+b1".parse::<Version>().unwrap().increment_bin_nmu()
+        );
+        assert_eq!(
+            "1.0+b1".parse::<Version>().unwrap(),
+            "1.0".parse::<Version>().unwrap().increment_bin_nmu()
+        );
+        assert_eq!(
+            "1.0-1+b1".parse::<Version>().unwrap(),
+            "1.0-1".parse::<Version>().unwrap().increment_bin_nmu()
+        );
+    }
+
+    #[test]
+    fn test_nmu_count() {
+        assert_eq!(Some(1), "1.0+nmu1".parse::<Version>().unwrap().nmu_count());
+        assert_eq!(
+            Some(1),
+            "1.0-1+nmu1".parse::<Version>().unwrap().nmu_count()
+        );
+        assert_eq!(None, "1.0-1".parse::<Version>().unwrap().nmu_count());
+        assert_eq!(None, "1.0".parse::<Version>().unwrap().nmu_count());
+    }
+
+    #[test]
+    fn test_is_nmu() {
+        assert!("1.0+nmu1".parse::<Version>().unwrap().is_nmu());
+        assert!("1.0-1+nmu1".parse::<Version>().unwrap().is_nmu());
+        assert!(!"1.0-1".parse::<Version>().unwrap().is_nmu());
+        assert!(!"1.0".parse::<Version>().unwrap().is_nmu());
+    }
+
+    #[test]
+    fn test_comparing_very_long_versions() {
+        // These are actual version numbers seen in actual apt repositories
+        let a = "1:11.1.0~++20210314110124+1fdec59bffc1-1~exp1~20210314220751.162";
+        let b = "1:11.1.0~++20211011013104+1fdec59bffc1-1~exp1~20211011133507.6";
+        assert_cmp!(a, b, Less);
+    }
+
+    #[test]
+    fn test_drop_leading_zeroes() {
+        use super::drop_leading_zeroes;
+
+        // Test basic cases
+        assert_eq!(drop_leading_zeroes("1.0"), "1.0");
+        assert_eq!(drop_leading_zeroes("001.0"), "1.0");
+        assert_eq!(drop_leading_zeroes("000.1"), "0.1");
+
+        // Test edge cases for missed mutants
+        assert_eq!(drop_leading_zeroes("0"), "0");
+        assert_eq!(drop_leading_zeroes("00"), "0");
+        assert_eq!(drop_leading_zeroes("0a"), "0a");
+        assert_eq!(drop_leading_zeroes("01a"), "1a");
+
+        // Test single character
+        assert_eq!(drop_leading_zeroes("a"), "a");
+
+        // Test empty string
+        assert_eq!(drop_leading_zeroes(""), "");
+    }
+
+    #[test]
+    fn test_version_cmp_part_edge_cases() {
+        // Test cases that should hit the missed mutants in version_cmp_part
+
+        // Test empty strings
+        assert_eq!(version_cmp_part("", ""), Ordering::Equal);
+        assert_eq!(version_cmp_part("", "1"), Ordering::Less);
+        assert_eq!(version_cmp_part("1", ""), Ordering::Greater);
+
+        // Test digit comparison edge cases
+        assert_eq!(version_cmp_part("1", "1"), Ordering::Equal);
+        assert_eq!(version_cmp_part("01", "1"), Ordering::Equal);
+
+        // Test very long digit sequences to hit BigInt path
+        let long_a = "123456789012345678901234567890";
+        let long_b = "123456789012345678901234567891";
+        assert_eq!(version_cmp_part(long_a, long_b), Ordering::Less);
+
+        // Test mixed digit/non-digit sequences
+        assert_eq!(version_cmp_part("1a2", "1a3"), Ordering::Less);
+        assert_eq!(version_cmp_part("1a2", "1b1"), Ordering::Less);
+
+        // Test tilde handling
+        assert_eq!(version_cmp_part("1~", "1"), Ordering::Less);
+        assert_eq!(version_cmp_part("~1", "1"), Ordering::Less);
+    }
+
+    #[test]
+    fn test_canonicalize_edge_cases() {
+        // Test cases that should hit missed mutants in canonicalize
+
+        // Test epoch handling
+        let v1 = Version {
+            epoch: Some(0),
+            upstream_version: "1.0".to_string(),
+            debian_revision: None,
+        };
+        let canonical = v1.canonicalize();
+        assert_eq!(canonical.epoch, None);
+
+        // Test debian revision all zeros
+        let v2 = Version {
+            epoch: None,
+            upstream_version: "1.0".to_string(),
+            debian_revision: Some("000".to_string()),
+        };
+        let canonical2 = v2.canonicalize();
+        assert_eq!(canonical2.debian_revision, None);
+
+        // Test debian revision with leading zeros
+        let v3 = Version {
+            epoch: None,
+            upstream_version: "1.0".to_string(),
+            debian_revision: Some("001".to_string()),
+        };
+        let canonical3 = v3.canonicalize();
+        assert_eq!(canonical3.debian_revision, Some("1".to_string()));
+
+        // Test upstream version with leading zeros
+        let v4 = Version {
+            epoch: None,
+            upstream_version: "001.0".to_string(),
+            debian_revision: None,
+        };
+        let canonical4 = v4.canonicalize();
+        assert_eq!(canonical4.upstream_version, "1.0");
+
+        // Test unchanged cases
+        let v5 = Version {
+            epoch: Some(1),
+            upstream_version: "1.0".to_string(),
+            debian_revision: Some("1".to_string()),
+        };
+        let canonical5 = v5.canonicalize();
+        assert_eq!(canonical5.upstream_version, "1.0");
+        assert_eq!(canonical5.debian_revision, Some("1".to_string()));
+    }
+
+    #[test]
+    fn test_partial_eq_false() {
+        // Test PartialEq returning false to catch the missed mutant
+        assert!("1.0-1"
+            .parse::<Version>()
+            .unwrap()
+            .ne(&"1.0-2".parse::<Version>().unwrap()));
+
+        assert!("1.0-1"
+            .parse::<Version>()
+            .unwrap()
+            .ne(&"2.0-1".parse::<Version>().unwrap()));
+
+        assert!("1:1.0-1"
+            .parse::<Version>()
+            .unwrap()
+            .ne(&"2:1.0-1".parse::<Version>().unwrap()));
+    }
+
+    #[test]
+    fn test_non_digit_cmp_edge_cases() {
+        use super::non_digit_cmp;
+
+        // Test tilde vs regular chars
+        assert_eq!(non_digit_cmp("~", "a"), Ordering::Less);
+        assert_eq!(non_digit_cmp("~", "A"), Ordering::Less);
+        assert_eq!(non_digit_cmp("~", "!"), Ordering::Less);
+
+        // Test special character ordering
+        assert_eq!(non_digit_cmp("!", "@"), Ordering::Less);
+        assert_eq!(non_digit_cmp("@", "A"), Ordering::Greater);
+        assert_eq!(non_digit_cmp("Z", "["), Ordering::Less);
+
+        // Test empty strings
+        assert_eq!(non_digit_cmp("", ""), Ordering::Equal);
+        assert_eq!(non_digit_cmp("", "a"), Ordering::Less);
+        assert_eq!(non_digit_cmp("a", ""), Ordering::Greater);
+    }
 }
